#include "alias.h"

#include "algorithms.h"

#include <fstream>
using namespace std;

//////// disk access ////////

disk_name		diskname;
alias_data		diskdata;
info_snapshot	readdata;

bool save_aliases(char *fn)
{
	ofstream file(fn, ios::binary);
	if (!file) return false;

	_listnode <alias_name> *parse = AliasList.head;

	while (parse)
	{
		alias_name *alias = parse->item;

		int count = 0;
		for (int i = 0; i < MEMORY_LENGTH; ++i)
		{
			alias_data *data = alias->data[i];
			if (!data) break;
			++count;
		}

		strncpy(diskname.name, alias->name, 24);
		diskname.seen = alias->seen;
		diskname.timediff = alias->timediff;
		diskname.data_count = count;

		if (!file.write((char*)&diskname, sizeof(diskname)))
			return false;

		for (int i = 0; i < count; ++i)
		{
			alias_data *data = alias->data[i];

			if (!file.write((char*)data, sizeof(alias_data)))
				return false;
		}

		parse = parse->next;
	}

	return true;
}

bool load_aliases(char *fn)
{
	ifstream file(fn, ios::binary);
	if (!file) return false;

    while (file.read((char*)&diskname, sizeof(diskname)))
	{
		alias_name *name = new alias_name;

		strncpy(name->name, diskname.name, 24);
		name->name_hash = get_name_hash(diskname.name);
		name->seen = diskname.seen;
		name->timediff = diskname.timediff;
		name->data[0] = NULL;

		AliasList.append(name);

		for (int i = 0; i < diskname.data_count; ++i)
		{
			if (!file.read((char*)&diskdata, sizeof(diskdata)))
				return false;

			readdata.id = diskdata.id;
			readdata.ip = diskdata.ip;
			readdata.tzb = diskdata.tzb;

			AddData(name, readdata);
		}
	}

	return true;
}


//////// utils ////////

Uint32 get_name_hash(const char *name)
{
	Uint32 b, hash = 0;

	while (b = *name++)
	{
		hash = ROR(hash, 5) + b;
	}

	return hash;
}

Uint32 get_ip(const char *cp)
{
	Uint32 ip = 0;

	char c, buffer[32];
	int i = 0;

	do
	{
		c = *cp++;

		if ((c == '.') || (c == '\0'))
		{
			buffer[i] = '\0';
			i = 0;

			Uint32 node = getInteger(buffer, 10);

			ip >>= 8;
			ip |= (node & 255) << 24;
		}
		else
			buffer[i++] = c;
	} while (c);

	return ip;
}


//////// Alias list ////////

_linkedlist <alias_name> AliasList;

bool ListChanged = false;

alias_name *AddAlias(info_snapshot &snapshot)
{
	alias_name *name = new alias_name;

	strncpy(name->name, snapshot.name, 24);
	name->data[0] = NULL;

	AliasList.append(name);

	return name;
}

bool old_info(alias_name *name, info_snapshot &snapshot)
{
	bool ip_match = false;
	bool id_match = false;

	for (int i = 0; i < MEMORY_LENGTH; ++i)
	{
		alias_data *data = name->data[i];
		if (!data) break;

		if ((snapshot.ip & 0x0000ffff) == (data->ip & 0x0000ffff))
		{
			ip_match = true;
		}

		if (snapshot.id == data->id)
		{
			id_match = true;
		}
	}

	return (ip_match && id_match);
}

void AddData(alias_name *name, info_snapshot &snapshot)
{
	for (int i = 0; i < MEMORY_LENGTH; ++i)
	{
		if (!name->data[i]) break;
	}

	alias_data *data = new alias_data;

	data->id = snapshot.id;
	data->ip = snapshot.ip;
	data->tzb = snapshot.tzb;
	name->timediff = snapshot.timediff;

	if (i == MEMORY_LENGTH)
	{
		// no more room
		for (int i = 3; i < MEMORY_LENGTH - 1; ++i)
		{
			name->data[i] = name->data[i + 1];
		}

		name->data[MEMORY_LENGTH - 1] = data;
	}
	else
	{
		// append

		name->data[i++] = data;

		if (i != MEMORY_LENGTH)
			name->data[i] = NULL;
	}

	ListChanged = true;
}

void UpdateHash(alias_name &snapshot)
{
	tolower(snapshot.name);
	snapshot.name_hash = get_name_hash(snapshot.name);
}

void UpdateSeen(alias_name &snapshot)
{
	time(&snapshot.seen);
}

alias_name *FindSnapshot(char *name)
{
	_listnode <alias_name> *parse = AliasList.head;

	tolower(name);
	Uint32 hash = get_name_hash(name);

	while (parse)
	{
		alias_name *alias = parse->item;

		if (hash == alias->name_hash)
		if (CMPSTR(name, alias->name))
			return alias;

		parse = parse->next;
	}

	return NULL;
}

String GetLastSeen(char *name)
{
	alias_name *alias = FindSnapshot(name);

	if (alias)
	{
		return asctime(gmtime(&alias->seen));
	}
	else
	{
		return "Haven't seen them.";
	}
}
