#include "types.h"
using namespace cat;
#include <time.h>
#include <iostream>
#include <list>
#include "alias.h"
#define STRING_CAST_CHAR
#include "datatypes.h"
#include <time.h>

bool LoadSSBotAliases(char *fn)
{
	ifstream file(fn);
	if (!file) return false;

	String mid, ip, tzb, pname;

	cout << "Loading aliases...";

	char line[1001];

	while (file.getline(line, 1000))
	{
		pname = line;
		mid = pname.split(':');
		ip = pname.split(':');
		tzb = pname.split(':');

		alias_name *alias = FindSnapshot(pname.msg);
		info_snapshot snapshot;

		strncpy(snapshot.name, pname.msg, 24);
		snapshot.seen = 0;
		snapshot.timediff = 0;
		snapshot.tzb = tzb.toInteger();
		snapshot.id = mid.toInteger();
		snapshot.ip = get_ip(ip.msg);

		if (!alias)
		{
			alias = AddAlias(snapshot);

			UpdateHash(*alias);

			AddData(alias, snapshot);
			alias->seen = 0;
		}
		else
		{
			if (!old_info(alias, snapshot))
			{
				AddData(alias, snapshot);
			}
		}
	}

	cout << "Done loading aliases.\n";

	return true;
}

bool LoadSSBotSeen(char *fn)
{
	ifstream file(fn);
	if (!file) return false;

	String pname, month, day, year, hour, minute, seconds, ampm;

	cout << "Loading seen...";

	char line[1001];

	file.getline(line, 1000); // skip [Seen]

	while (file.getline(line, 1000))
	{
		ampm = line;
		pname = ampm.split('=');
		month = ampm.split('/');
		if (month == ampm)
			month = ampm.split('.');
		day = ampm.split('/');
		if (day == ampm)
			day = ampm.split('.');
		year = ampm.split('&');
		if (year == ampm)
			year = ampm.split(' ');
		hour = ampm.split(':');
		minute = ampm.split(':');
		seconds = ampm.split(' ');

		alias_name *alias = FindSnapshot(pname.msg);
		if (alias)
		{
			tm tt;
			tt.tm_hour = hour.toInteger() - 5; // for some reason this was messed
			tt.tm_mday = day.toInteger();
			tt.tm_min = minute.toInteger();
			tt.tm_mon = month.toInteger() - 1;
			tt.tm_sec = seconds.toInteger();
			tt.tm_year = year.toInteger();

			if (tt.tm_year < 100)
				tt.tm_year += 2000;

			tt.tm_year -= 1900;

			tt.tm_isdst = false;
			tt.tm_wday = 0;
			tt.tm_yday = 0;

			if (ampm == "PM")
				tt.tm_hour += 12;

			time_t seen = mktime(&tt);
//jente!!!=12/12/01 4:27:33 PM
/*			if (pname == "jente!!!")
			{
				cout << pname.msg << "\n";
				cout << tt.tm_mon << "\n";
				cout << tt.tm_mday << "\n";
				cout << tt.tm_year << "\n";
				cout << tt.tm_hour << "\n";
				cout << tt.tm_min << "\n";
				cout << tt.tm_sec << "\n";
				cout << tt.tm_wday << "\n";
				cout << tt.tm_yday << "\n";
			}
*/
			if (alias->seen < seen)
			{
				alias->seen = seen;
			}
		}
	}

	cout << "Done loading seen.\n";

	return true;
}

void PrintBanner()
{
	cout << "Alias.DAT Merger (BUILD 1) : SS-Bot -> Alias.DLL\n";
	cout << " by cat02e@fsu.edu\n";
}

void PrintHelp(char *myname)
{
	cout << "\n";
	cout << "Usage:\n";
	cout << " " << myname << " <SSBot Alias.DAT file> <SSBot Seen.DAT file> <MERVBot Alias.DAT file> <Output Alias.DAT file>\n";
}

int main(int argc, char *argv[])
{
	if (argc != 5)
	{
		if (argc > 0)
			PrintHelp(argv[0]);
		else
			PrintHelp("CONVALIAS.EXE");
		return 0;
	}

	char *ssbot_dat_file = argv[1];
	char *seen_dat_file = argv[2];
	char *mervbot_dat_file = argv[3];
	char *output_dat_file = argv[4];

	if (!load_aliases(mervbot_dat_file))
	{
		cout << "Error:\n";
		cout << " Unable to load MERVBot dat file: " << mervbot_dat_file << "\n";
		cout << " Continuing with conversion...\n";
	}

	if (!LoadSSBotAliases(ssbot_dat_file))
	{
		cout << "Error:\n";
		cout << " Unable to load SS-Bot dat file: " << ssbot_dat_file << "\n";
		cout << " Unable to continue with conversion of alias data...\n";
	}

	if (!LoadSSBotSeen(seen_dat_file))
	{
		cout << "Error:\n";
		cout << " Unable to load SS-Bot dat file: " << seen_dat_file << "\n";
		cout << " Unable to continue with conversion of seen data...\n";
	}

	if (!save_aliases(output_dat_file))
	{
		cout << "Error:\n";
		cout << "Unable to save MERVBot dat file: " << output_dat_file << "\n";
	}

	return 0;
}
